/*/
      Copyright (c) 1990 Optimal Software
/*/

/*/
      This file contains the functions which interface with the operating
      system memory manager.  The functions implement malloc, free, and
      realloc using paragraph-oriented pointers and sizes.

      The functions are clones of the MSC 5.1 run-time library routines
      _dos_allocmem, _dos_freemem, and _dos_setblock.  The argument lists
      and return values are identical.
/*/

#include <dos.h>

#include "heap.h"

#define DOS_MALLOC  0x48   /* Int 21 function codes */
#define DOS_FREE    0x49
#define DOS_REALLOC 0x4A

#define EDOSBADMEM 7    /* memory headers trashed */
#define EDOSNOMEM  8    /* insufficient memory */
#define EDOSBADSEG 9    /* invalid segment number */

/******************************************************************************/
/*/
      Get memory from DOS

      Returns zero if successful, otherwise a DOS error code.  If successful
      it stores the segment pointer in *segment.  If unsuccessful it stores
      the maximum size available in *segment.
/*/
extern int pascal _dos_malloc( uint size, uint *segment )
{
int result  = 0;

union REGS regs;

regs.h.ah = DOS_MALLOC;
regs.x.bx = size;

intdos( &regs, &regs );

if ( regs.x.cflag )              /* function failed; investigate further */
   {
   result = regs.x.ax;           /* dos error code */

   *segment = result == EDOSNOMEM ? regs.x.bx      /* space available */
                                  : 0;             /* no space left */
   }
else

   *segment = regs.x.ax;         /* segment address of allocated memory */

return( result );
}
/******************************************************************************/
/*/
      Release memory to DOS

      Returns zero if successful, otherwise a DOS error code
/*/
extern int pascal _dos_free( uint segment )
{
int result = 0;

union REGS regs;

struct SREGS sregs;

regs.h.ah = DOS_FREE;
sregs.es  = segment;

intdosx( &regs, &regs, &sregs );

if ( regs.x.cflag )        /* function failed; return DOS error code */

   result = regs.x.ax;

return( result );
}
/******************************************************************************/
/*/
      Adjust a DOS memory block

      Returns zero if successful, otherwise a DOS error code.  If unsuccessful
      it stores the maximum size available in *limit.
/*/
extern int pascal _dos_realloc( uint size, uint segment, uint *limit )
{
int result = 0;

union REGS regs;

struct SREGS sregs;

regs.h.ah = DOS_REALLOC;
regs.x.bx = size;
sregs.es  = segment;

intdosx( &regs, &regs, &sregs );

if ( regs.x.cflag )              /* function failed; investigate further */
   {
   result = regs.x.ax;           /* dos error code */

   *limit = result == EDOSNOMEM ? regs.x.bx   /* space available */
                                : 0;          /* no space left */
   }

return( result );
}
/******************************************************************************/
